﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.CombinedIntegerVectorEncoding {
  [Item("UniformOnePositionManipulator", "")]
  [StorableClass]
  public class UniformOnePositionInConditionManipulator : CombinedIntegerVectorManipulator {

    [StorableConstructor]
    protected UniformOnePositionInConditionManipulator(bool deserializing) : base(deserializing) { }
    protected UniformOnePositionInConditionManipulator(UniformOnePositionInConditionManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformOnePositionInConditionManipulator(this, cloner);
    }
    public UniformOnePositionInConditionManipulator()
      : base() {
    }

    protected override CombinedIntegerVector Manipulate(IRandom random, CombinedIntegerVector input, CombinedIntegerVector child) {
      return Manipulate(input, child, random.Next(((CombinedIntegerVector)child.Condition).Length));
    }

    public static CombinedIntegerVector Manipulate(CombinedIntegerVector input, CombinedIntegerVector child, int index) {
      int max = child.Bounds[index % child.Bounds.Rows, 1] - 1;
      if (child[index].Equals(max)) {
        child[index] = input[index];
      } else {
        child[index] = max;
      }
      return child;
    }
  }
}
